from . import utils
import dataclasses
import strawberry
import typing
import django.core.exceptions

lookups = ('exact', 'iexact', 'contains', 'icontains', 'in',
        'gt', 'gte', 'lt', 'lte',
        'startswith', 'istartswith', 'endswith', 'iendswidth',
        'range', 'isnull', 'regex', 'iregex')

#LookupValueType = typing.Optional[typing.Any]
#LookupValueType = typing.Union[jstr, int, None]

def get_field_type(lookup, field_type):
    if lookup in ('in', 'range'):
        field_type = typing.List[field_type]
    elif lookup in ('regex', 'iregex'):
        field_type = str
    elif lookup in ('isnull'):
        field_type = bool
    return typing.Optional[field_type]

def gen_field_lookups(field_name, field_type):
    fields = [ (f'{field_name}__{lookup}', get_field_type(lookup, field_type)) for lookup in lookups ]
    return fields

def gen_filter_type(object_type):
    global cls
    model = object_type._django_model
    type_name = f'{model._meta.object_name}FilterType'
    fields = [
    #    ('OR', cls),
    #    ('OR', type_name),
    #    ('NOT', type_name),
    ]
    for field in dataclasses.fields(object_type):
        try:
            model_field = model._meta.get_field(field.name)
        except django.core.exceptions.FieldDoesNotExist:
            continue
        if model_field.is_relation:
            continue

        fields += gen_field_lookups(field.name, field.type)

    cls = dataclasses.make_dataclass(type_name, fields)
    #cls.__annotations__['OR'] = typing.Optional[typing.List[cls]]
    #cls.__annotations__['NOT'] = typing.Optional[typing.List[cls]]
    #cls.__annotations__['OR'] = typing.Optional['cls']
    cls = strawberry.input(cls)
    return cls






import strawberry
import typing
from .types import get_model_fields, update_fields

_type = type

def type(model, *, fields=None, types=None, is_update=False, **kwargs):
    def wrapper(cls):
        is_input = kwargs.get('is_input', False)
        model_fields = get_model_fields(cls, model, fields, types, is_input, is_update)
        if not hasattr(cls, '__annotations__'):
            cls.__annotations__ = {}
        for field_name, field_type, field_value in model_fields:
            cls.__annotations__[field_name] = field_type
            setattr(cls, field_name, field_value)
        update_fields(cls, model)
        cls._django_model = model
        cls._is_update = is_update
        return strawberry.type(cls, **kwargs)
    return wrapper


def input(model, *, fields=None, types=None, is_update=False, **kwargs):
    return type(model, fields=fields, types=types, is_update=is_update, is_input=True, **kwargs)


def generate_update_from_input(model, input):
    if input._is_update:
        return input
    cls = _type(f'{input.__name__}Update', (), { '__annotations__': {}})
    for field_name, field_type in input.__annotations__.items():
        field_value = getattr(input, field_name)
        if typing.get_origin(field_type) != typing.Optional:
            field_type = typing.Optional[field_type]
        cls.__annotations__[field_name] = field_type
        setattr(cls, field_name, field_value)
    cls._django_model = model
    return strawberry.type(cls, is_input=True)
